package net.sf.javaprinciples.resource;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.activation.MimetypesFileTypeMap;

/**
 * 
 * @author wslade
 *
 */
public class FilesystemResource extends AbstractResource<String>
{
    private File file;
    private ResourceMetadata metadata;

    /**
     * @return a new FileInputStream which must be closed by the caller.
     * @throws IOException if the file fails to open
     */
    @Override
    public InputStream getInputStream() throws IOException
    {
        return new FileInputStream(file);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public long getResourceLength()
    {
        if (metadata != null)
        {
            return metadata.getContentLength();
        }

        return file.length();
    }

    /**
     * @param identifier The identifier of a resource.
     */
    @Override
    public void setContent(String identifier)
    {
        // ignore;
    }

    /**
     * Assign the filesystem file to the resource.
     * 
     * Determine the mime type.
     * @param file A non null file descriptor.
     */
    public void setFile(File file)
    {
        this.file = file;
        
        mimeType = new MimetypesFileTypeMap().getContentType(file);
    }
    
    /**
     * Retrieves file from resource.
     * 
     * @return file A non null file descriptor.
     */
    public File getFile()
    {
        return this.file;
    }

    public void setMetadata(ResourceMetadata metadata)
    {
        this.metadata = metadata;

        mimeType = metadata.getContentType();
        validFrom = metadata.getCreatedDate().getTime();
        validUntil = metadata.getExpiryDate().getTime();
        encoding = metadata.getFileEncoding();
    }
}
