package net.sf.javaprinciples.resource;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.List;
import java.util.Map;

public class SimpleResourceIdentifier implements ResourceIdentifier
{
    private String identifier;

    /**
     * Create a ResourceIdentifier.
     *
     * @param identifier Just the last part of the URL
     */
    public SimpleResourceIdentifier(String identifier)
    {
        this.identifier = identifier;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String getIdentifier()
    {
        return identifier;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<String> getPathElements()
    {
        throw new RuntimeException("Simple resource should not be when the path is required.");
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Map<String, String> getQueryParameters()
    {
        throw new RuntimeException("Simple resource should not be when the query is required.");
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString()
    {
        try
        {
            return URLEncoder.encode(identifier, "UTF-8");
        }
        catch (UnsupportedEncodingException uee)
        {
            return "";
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean equals(Object other)
    {
        if (!(other instanceof SimpleResourceIdentifier))
        {
            return false;
        }
        SimpleResourceIdentifier toCompare = (SimpleResourceIdentifier)other;
        return identifier.equals(toCompare.identifier);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public int hashCode()
    {
        // This sort of thing can be extracted into a utility class.
        return identifier.hashCode();
    }

}
