package net.sf.javaprinciples.resource;

import java.net.URISyntaxException;

import net.sf.javaprinciples.core.UnexpectedException;

public class UrnFileResolver extends RootFileResolver
{
    private String suffix;
    private boolean safeNames = true;

    protected String makeFilename(ResourceIdentifier identifier)
    {
        try
        {
            URN urn = new URN(identifier.getIdentifier());
            String filename = safeNames ? urn.getSafeString() : urn.toString();
            if (suffix != null)
            {
                return filename + suffix;
            }
            return filename;
        }
        catch (URISyntaxException e)
        {
            throw new UnexpectedException("Provided identifier is not a well formed URN.", e);
        }
    }

    public void setSuffix(String suffix)
    {
        this.suffix = suffix;
    }

    public String getSuffix()
    {
        return suffix;
    }

    public void setSafeNames(boolean safeNames)
    {
        this.safeNames = safeNames;
    }

    public boolean getSafeNames()
    {
        return safeNames;
    }
}
