package net.sf.javaprinciples.resource.servlet;

import java.lang.reflect.InvocationTargetException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;
import org.springframework.web.util.UrlPathHelper;

import net.sf.javaprinciples.resource.ResourceSecurity;
import net.sf.javaprinciples.resource.ResourceSecurityException;

public class ResourceSecurityInterceptor extends HandlerInterceptorAdapter
{
    private static final Log LOGGER = LogFactory.getLog(ResourceSecurityInterceptor.class);
    private String parameterName = "key";
    private String requestProperty = "referer";

    private String[] securedServices;

    private ResourceSecurity security;

    private UrlPathHelper urlPathHelper = new UrlPathHelper();

    /**
     *
     * @param securedServices Never null
     */
    public void setSecuredServices(String[] securedServices)
    {
        this.securedServices = securedServices;
    }

    /**
     *
     * @return Only null when not injected
     */
    public String[] getSecuredServices()
    {
        return securedServices;
    }

    /**
     *
     * @param security Never null
     */
    public void setSecurity(ResourceSecurity security)
    {
        this.security = security;
    }

    /**
     *
     * @return Only null when not injected
     */
    public ResourceSecurity getSecurity()
    {
        return security;
    }

    /**
     * @param request Never null
     * @param response Never null
     * @param handler Never null
     * @return true when processing should continue
     * @throws Exception never
     */
    public boolean preHandle(HttpServletRequest request,
            HttpServletResponse response, Object handler) throws Exception
    {
        String uri = urlPathHelper.getLookupPathForRequest(request);
        if (applies(uri))
        {
            try
            {
                check(request);
            }
            catch (ResourceSecurityException e)
            {
                response.setStatus(HttpServletResponse.SC_FORBIDDEN);
                return false;
            }
        }
        return true;
    }

    protected boolean applies(String uri)
    {
        for (String securedService : securedServices)
        {
            if (uri.startsWith(securedService))
            {
                return true;
            }
        }
        return false;
    }

    protected void check(HttpServletRequest request)
            throws ResourceSecurityException
    {
        String parameter = request.getParameter(parameterName);
        // Attempt to get the property from the header
        String property = request.getHeader(requestProperty);
        if (property == null)
        {
            try
            {
                property = (String) PropertyUtils.getProperty(request, requestProperty);
            }
            catch (IllegalAccessException e)
            {
                LOGGER.info("IllegalAccessException: For property " + requestProperty);
            }
            catch (InvocationTargetException e)
            {
                LOGGER.info("InvocationTargetException: For property " + requestProperty);
            }
            catch (NoSuchMethodException e)
            {
                LOGGER.info("NoSuchMethodException: For property " + requestProperty);
            }
        }
        security.checkSecurity(parameter, property);
    }

    /**
     * @param parameterName the parameterName to set
     */
    public void setParameterName(String parameterName)
    {
        this.parameterName = parameterName;
    }

    /**
     * @param requestProperty the requestProperty to set
     */
    public void setRequestProperty(String requestProperty)
    {
        this.requestProperty = requestProperty;
    }
}
